package fr.emse.gitlab.saref.jobs.library;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.riot.Lang;

import fr.emse.gitlab.saref.jobs.AbstractJobRunner;

public class ReadOntologies extends AbstractJobRunner {

	public ReadOntologies(File dir) {
		super(dir);
	}	
	
	@Override
	protected void doJob0() {
		String repoName = getRepoName();
		String ontologyName = repoName.equals("saref-core") ? "saref.ttl" : repoName + ".ttl";
		checkOntology(ontologyName);
		checkExamples();
	}

	private void checkOntology(String ontologyName) {
		File ontologyFile = new File(directory, "ontology/" + ontologyName);
		Model model = ModelFactory.createDefaultModel();
		try(FileInputStream input = new FileInputStream(ontologyFile)) {
			model.read(input, null, Lang.TTL.getLabel());
		} catch ( Exception ex) {
			error("Exception while reading the ontology file", ex);
		}
	}

	private void checkExamples() {
		File dir = new File(directory, "examples");
		try {
			Files.walk(dir.toPath()).filter(p -> {
				return p.endsWith(".ttl");
			}).forEach(p->{
				File exampleFile = p.toFile();
				Model model = ModelFactory.createDefaultModel();
				try(FileInputStream input = new FileInputStream(exampleFile)) {
					model.read(input, null, Lang.TTL.getLabel());
				} catch ( Exception ex) {
					failure(String.format("Exception while reading the example file %s", exampleFile.toString()), ex.getClass().getSimpleName(), ex.getMessage());
				}
			});
		} catch(IOException ex) {
			error("Exception while walking the example directory", ex);
		}
	}

	private String getRepoName() {
		String REGEX = "^(?<ext>saref-core|saref4[a-z][a-z][a-z][a-z])";
		String name = directory.getName();
		Pattern pattern = Pattern.compile(REGEX);
		Matcher matcher = pattern.matcher(name);
		if (!matcher.find()) {
			error(String.format(
					"The SAREF pipeline must be run inside a directory whose name begins with `saref-core`, or `saref4abcd` (where abcd can be any sequence of four letters). Got: %s",
					name));
			return null;
		}
		return matcher.group("ext");
	}

}
